"use client";
import { useState } from "react";
import queryString from "query-string";
import { useToast } from "@/contexts/ToastContext";

type Method = "GET" | "POST" | "PUT" | "DELETE" | "PATCH";

type Params = Record<string, any>;

interface UseFetch {
    url: string;
    method: Method;
    cacheTime?: number;
}

interface BaseResponse<T> {
    data: T;
    code: number;
    message: string;
}
type Config =
    | { next: { revalidate: number } }
    | { cache: "no-store" }
    | { cache: "force-cache" };

const useFetch = <T,>({ url, method, cacheTime = 0 }: UseFetch) => {
    const [data, setData] = useState<any>(null);
    const [loading, setLoading] = useState<boolean>(false);
    const [error, setError] = useState<string | null>(null);

    const { addToast } = useToast();
    const fetchData = async (params?: Params): Promise<any> => {
        setError(null);
        setLoading(true);

        return new Promise(async (resolve, reject) => {
            try {
                const { url: requestUrl, options } = interceptorsRequest({
                    url,
                    method,
                    params,
                    cacheTime,
                });
                const response = await fetch(requestUrl, options);

                const result = await interceptorsResponse<BaseResponse<T>>(
                    response
                );
                console.log("response", response);
                console.log("result", result);
                setData(result.data);
                resolve(result.data); // 成功时 resolve 结果
            } catch (err: any) {
                setError(err.message);
                addToast(err.message, "error");
                reject(err.message); // 失败时 reject 错误信息
            } finally {
                setLoading(false);
            }
        });
    };

    return { fetchData, data, loading, error };
};

// 请求拦截器
const interceptorsRequest = ({
    url,
    method,
    params,
    cacheTime,
}: UseFetch & { params?: Params }) => {
    let queryParams = ""; // url 参数
    let requestPayload: any = ""; // 请求体数据

    const headers = {
        // authorization: `Bearer ...`,
    };

    const config: Config =
        cacheTime || cacheTime === 0
            ? cacheTime > 0
                ? { next: { revalidate: cacheTime } }
                : { cache: "no-store" }
            : { cache: "force-cache" };

    if (method === "GET" || method === "DELETE") {
        // GET/DELETE 请求不能将参数放在 body 中，只能拼接到 URL 上
        if (params) {
            queryParams = queryString.stringify(params);
            url = `${url}?${queryParams}`;
        }
    } else {
        // 非 form-data 传输 JSON 数据格式
        if (Object.prototype.toString.call(params) === "[object FormData]") {
            requestPayload = params; // 如果是 FormData 类型，不做处理
        } else if (
            !["[object FormData]", "[object URLSearchParams]"].includes(
                Object.prototype.toString.call(params)
            )
        ) {
            Object.assign(headers, { "Content-Type": "application/json" });
            requestPayload = JSON.stringify(params);
        }
    }

    return {
        url,
        options: {
            method,
            headers,
            body:
                method !== "GET" && method !== "DELETE"
                    ? requestPayload
                    : undefined,
            ...config,
        },
    };
};

// 响应拦截器
const interceptorsResponse = <T,>(res: Response): Promise<T> => {
    return new Promise((resolve, reject) => {
        const requestUrl = res.url;
        if (res.ok) {
            res.clone()
                .text()
                .then((text) => {
                    try {
                        const data = JSON.parse(text);
                        console.log("data", data);
                        if (!data.code) {
                            return resolve(res.json() as Promise<T>);
                        }
                        if (data.code === 200) {
                            return resolve(res.json() as Promise<T>);
                        } else if (data.code === 401) {
                            console.log("data.code", data.code);
                            window.location.href = "/login";
                            return reject({
                                message: data.message,
                                url: requestUrl,
                            });
                        } else {
                            return reject({
                                message: data.message,
                                url: requestUrl,
                            });
                        }
                    } catch {
                        console.log("catch");
                        return reject({ message: text, url: requestUrl });
                    }
                });
        } else {
            res.clone()
                .text()
                .then((text) => {
                    try {
                        const errorData = JSON.parse(text);
                        return reject({
                            message: errorData || "接口错误",
                            url: requestUrl,
                        });
                    } catch {
                        return reject({ message: text, url: requestUrl });
                    }
                });
        }
    });
};

export default useFetch;
